# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for billing commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import resources


PROJECTS_COLLECTION = 'cloudbilling.projects'
ACCOUNTS_COLLECTION = 'cloudbilling.billingAccounts'


def GetRegistry():
  registry = resources.REGISTRY.Clone()
  registry.RegisterApiByName('cloudbilling', 'v1')
  return registry


def ParseProject(project_id):
  return GetRegistry().Parse(project_id, collection=PROJECTS_COLLECTION)


def ParseAccount(account_id):
  return GetRegistry().Parse(account_id, collection=ACCOUNTS_COLLECTION)
