# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Resource parsing utilities for services."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.core import resources


def GetServiceNameFromArg(service):
  if not service:
    return None
  service_ref = resources.REGISTRY.Parse(
      service,
      collection='servicemanagement.services')
  return service_ref.serviceName


def GetOperationIdFromArg(operation):
  if not operation:
    return None
  operation_ref = resources.REGISTRY.Parse(
      operation,
      collection='servicemanagement.operations')
  return operation_ref.operationsId
