# ${utf_encoding}
# Copyright ${current_year} Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for the ${test_command_string} config export command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.tests.lib import cli_test_base
from googlecloudsdk.tests.lib import sdk_test_base
from googlecloudsdk.tests.lib import test_case
from googlecloudsdk.tests.lib.command_lib.util.declarative import test_base


class ConfigExportTest(test_base.ConfigExportTestMixin,
                       cli_test_base.CliTestBase, sdk_test_base.WithFakeAuth):

  def SetUp(self):
    command = '${test_command_string}'
    resource_args = ['${test_command_arguments}']${pylint_disable}
    self.ConfigureTests(
        track=calliope_base.ReleaseTrack.ALPHA,
        collection='${full_collection_name}',
        command=command,
        resource_args=resource_args)


if __name__ == '__main__':
  test_case.main()
