# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Distributed Cloud Edge Network interconnects API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.edge_cloud.networking import utils


class InterconnectsClient(object):
  """Client for private connections service in the API."""

  def __init__(self, client=None, messages=None):
    self._client = client or utils.GetClientInstance()
    self._messages = messages or utils.GetMessagesModule()
    self._service = self._client.projects_locations_zones_interconnects

  def GetStatus(self, interconnect_ref):
    """Get the status of a specified interconnect."""
    get_interconnect_status_req = self._messages.EdgenetworkProjectsLocationsZonesInterconnectsDiagnoseRequest(
        name=interconnect_ref.RelativeName())
    return self._service.Diagnose(get_interconnect_status_req)
