# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Completers to help with tab-completing."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.resource_manager import completers as resource_manager_completers
from googlecloudsdk.command_lib.util import completers
from googlecloudsdk.command_lib.util import parameter_info_lib


class SecretsCompleter(completers.ListCommandCompleter):
  """A completer for the secret resource argument."""

  def __init__(self,
               collection='secretmanager.projects.secrets',
               list_command='secrets list --uri',
               **kwargs):
    super(SecretsCompleter, self).__init__(
        collection=collection, list_command=list_command, **kwargs)

  def ParameterInfo(self, parsed_args, argument):
    return parameter_info_lib.ParameterInfoByConvention(
        parsed_args,
        argument,
        self.collection,
        updaters={
            'projectsId': (resource_manager_completers.ProjectCompleter, True)
        },
    )


class BetaSecretsCompleter(SecretsCompleter):

  def __init__(self, **kwargs):
    super(BetaSecretsCompleter, self).__init__(
        list_command='beta secrets list --uri', **kwargs)
