# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for Data Catalog search commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.data_catalog import search


def Search(args, version_label):
  """Search Data Catalog for entries, tags, etc that match a query."""
  client = search.SearchClient(version_label)
  return client.Search(
      args.query,
      args.include_gcp_public_datasets,
      args.include_organization_ids,
      args.restricted_locations,
      args.include_project_ids,
      args.order_by,
      args.page_size,
      args.limit
  )
