# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""PrivateCA resource completers."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.command_lib.util import completers


class LocationsCompleter(completers.ListCommandCompleter):
  """The location completer."""

  def __init__(self, **kwargs):
    super(LocationsCompleter, self).__init__(
        collection='privateca.projects.locations',
        list_command='beta privateca locations list --uri',
        **kwargs)


class CertificatesCompleter(completers.ListCommandCompleter):
  """The certificate completer."""

  def __init__(self, **kwargs):
    super(CertificatesCompleter, self).__init__(
        collection='privateca.projects.locations.certificateAuthorities.certificates',
        list_command='beta privateca certificates list --uri',
        **kwargs)
