# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Cloud Datacatalog crawlers client."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.data_catalog import util

_CRAWLER_API_VERSION = 'v1alpha3'


class CrawlersClient(object):
  """Cloud Datacatalog crawlers client."""

  def __init__(self):
    self.client = util.GetClientInstance(api_version=_CRAWLER_API_VERSION)
    self.messages = util.GetMessagesModule(api_version=_CRAWLER_API_VERSION)
    self.service = self.client.projects_crawlers

  def Get(self, crawler_ref):
    request = self.messages.DatacatalogProjectsCrawlersGetRequest(
        name=crawler_ref.RelativeName())
    return self.service.Get(request)
